#include "math.h"
#include "mex.h"

double getRectangleSingleHaarFeature(double * rect, mxArray * iiData,double * ft_haar, double s, int u , int v);

void mexFunction(int nlhs, mxArray *plhs[ ],int nrhs, const mxArray *prhs[ ]) {

	// input data
	mxArray * rectData, * iiData, * sData, * index_HaarData;
	double * rectValues, * index_HaarValues;
	// output data
	double * haar_data_values;

	double * ft_haar,s;
	int u,v,n_fts,i,j,n_elem;

	// get input arguments
	rectData = prhs[0];
	iiData = prhs[1];
	index_HaarData = prhs[2];
	sData = prhs[3];

	rectValues = mxGetPr(rectData);
	index_HaarValues = mxGetPr(index_HaarData);

	s = (int)(mxGetScalar(sData));

	u = mxGetDimensions(iiData)[0];
	v = mxGetDimensions(iiData)[1];

	n_fts = mxGetDimensions(index_HaarData)[0];
	n_elem = mxGetDimensions(index_HaarData)[1];

	// allocate output data
	plhs[0] = mxCreateDoubleMatrix(n_fts, 1, mxREAL); //mxReal is our data-type
	//Get a pointer to the data space in our newly allocated memory
	haar_data_values = mxGetPr(plhs[0]);
	
	ft_haar = malloc(sizeof(double)*n_elem);


	for(i=0;i<n_fts;i++)
	{
		for(j=0;j<n_elem;j++)
        {
			ft_haar[j] = index_HaarValues[(j*n_fts)+i];
        }

		haar_data_values[i] = 
            getRectangleSingleHaarFeature(rectValues,iiData,ft_haar,s,u,v);
	}

	free(ft_haar);

	return;
}


double getRectangleSingleHaarFeature(double * rect, mxArray * iiData,double * ft_haar, double s, int u , int v)
{
// hh : histogram integrale
// ft_HoG
// ws : window size
	double * ii, sc, fx;
	int wnd_size,pas,x,y,dx, type;
	int A,B,C,D,E,F,G,H,mx;

	mx = u * v - 1;
    sc=ft_haar[2];
	wnd_size = 32;
	pas = rect[2] / wnd_size;
    type = ft_haar[1];
// Feature 
	x=rect[1]-1+(ft_haar[3]-1)*pas;
	y=rect[0]-1+(ft_haar[4]-1)*pas;
	dx = sc * pas;

	ii = mxGetPr(iiData);
    
    switch(type)
    {
        case 1:
    
            fx = ii[y+x*u] + 2 * ii[(y+dx)+(x+dx)*u] + ii[y+(x+2*dx)*u] - 
                2 * ii[y+(x+dx)*u] - ii[(y+dx)+x*u] - ii[(y+dx)+(x+2*dx)*u];
            break;
        case 2:
            fx = ii[y+x*u] + 2 * ii[(y+dx)+(x+dx)*u] + ii[(y+2*dx)+x*u] - 
                2 * ii[(y+dx)+x*u] - ii[y+(x+dx)*u] - ii[(y+2*dx)+(x+dx)*u];
            break;
		case 3:
			C = (y+dx)+(x+3*dx)*u;
			B = y+(x+3*dx)*u;
			A = y+x*u;
			D = (y+dx)+x*u;
			E = y+(x+dx+1)*u;
			F = y+(x+2*dx)*u;
			G = (y+dx)+(x+2*dx)*u;
			H = (y+dx)+(x+dx+1)*u;

			if (C > mx)
			{
				mexPrintf("f:%i , u: %i, v: %i, x: %i, y: %i, dx: %i, C: %i y max: %i\n",type,u,v,x,y,dx,C,mx);
				C = mx;
			}
			if (B > mx)
			{
				mexPrintf("u: %i, x: %i, y: %i, dx: %i, B: %i y max: %i\n",u,x,y,dx,B,mx);
				B = mx;
			}
			if (D > mx)
			{
				mexPrintf("u: %i, x: %i, y: %i, dx: %i, D: %i y max: %i\n",D,mx);
				D = mx;
			}
			if (E > mx)
			{
				mexPrintf("valor E: %i y max: %i\n",E,mx);
				E = mx;
			}
			if (F > mx)
			{
				mexPrintf("u: %i, x: %i, y: %i, dx: %i, F: %i y max: %i\n",u,x,y,dx,F,mx);
				F = mx;
			}
			if (G > mx)
			{
				mexPrintf("u: %i, x: %i, y: %i, dx: %i, G: %i y max: %i\n",u,x,y,dx,G,mx);
				G = mx;
			}
			if (H > mx)
			{
				mexPrintf("valor H: %i y max: %i\n",H,mx);
				H = mx;
			}



			fx = (ii[A] + ii[C] - (ii[B] + ii[D])) - 3 * (ii[G] + ii[E] - (ii[F] + ii[H]));

			break;
		case 4:
			A = y+x*u;
			D = (y+3*dx)+x*u;
			C = (y+3*dx)+(x+dx)*u;
			B = y+(x+dx)*u;
			E = (y+dx+1)+x*u;
			F = (y+dx+1)+(x+dx)*u;
			G = (y+2*dx)+(x+dx)*u;
			H = (y+2*dx)+x*u;

			if (C > mx)
			{
				mexPrintf("f:%i , u: %i, v: %i, x: %i, y: %i, dx: %i, C: %i y max: %i\n",type,u,v,x,y,dx,C,mx);
				C = mx;
			}
			if (B > mx)
			{
				mexPrintf("u: %i, x: %i, y: %i, dx: %i, B: %i y max: %i\n",u,x,y,dx,B,mx);
				B = mx;
			}
			if (D > mx)
			{
				mexPrintf("u: %i, x: %i, y: %i, dx: %i, D: %i y max: %i\n",D,mx);
				D = mx;
			}
			if (E > mx)
			{
				mexPrintf("valor E: %i y max: %i\n",E,mx);
				E = mx;
			}
			if (F > mx)
			{
				mexPrintf("u: %i, x: %i, y: %i, dx: %i, F: %i y max: %i\n",u,x,y,dx,F,mx);
				F = mx;
			}
			if (G > mx)
			{
				mexPrintf("u: %i, x: %i, y: %i, dx: %i, G: %i y max: %i\n",u,x,y,dx,G,mx);
				G = mx;
			}
			if (H > mx)
			{
				mexPrintf("valor H: %i y max: %i\n",H,mx);
				H = mx;
			}

			fx = (ii[A] + ii[C] - (ii[B] + ii[D])) - 3 * (ii[G] + ii[E] - (ii[F] + ii[H]));
			break;
    }
    fx = s > 0 ? fx / s : fx;
    
    fx=fabs(fx)/(pas*pas);
    
	return fx;
}
