#include "math.h"
#include "mex.h"

void getRectangleSingleHoGFeature(double * rect, mxArray * hhData,double * ft_hog, double * h, int N, int u , int v);

void mexFunction(int nlhs, mxArray *plhs[ ],int nrhs, const mxArray *prhs[ ]) {

	// input data
	mxArray * rectData, * hhData, * NData, * index_HogData;
	double * rectValues, * index_HogValues;
	// output data
	double * hog_data_values;

	double * h,* ft_hog;
	int u,v,n_fts,i,j,N,n_elem;

	// get input arguments
	rectData = prhs[0];
	hhData = prhs[1];
	NData = prhs[2];
	index_HogData = prhs[3];

	rectValues = mxGetPr(rectData);
	index_HogValues = mxGetPr(index_HogData);

	N = (int)(mxGetScalar(NData));

	u = mxGetDimensions(hhData)[0];
	v = mxGetDimensions(hhData)[1];

	n_fts = mxGetDimensions(index_HogData)[0];
	n_elem = mxGetDimensions(index_HogData)[1];

	// allocate output data
	plhs[0] = mxCreateDoubleMatrix(n_fts, N, mxREAL); //mxReal is our data-type
	//Get a pointer to the data space in our newly allocated memory
	hog_data_values = mxGetPr(plhs[0]);
	
	h = calloc(sizeof(double),N);
	ft_hog = malloc(sizeof(double)*n_elem);


	for(i=0;i<n_fts;i++)
	{
		for(j=0;j<n_elem;j++)
        {
			ft_hog[j] = index_HogValues[(j*n_fts)+i];
        }

		getRectangleSingleHoGFeature(rectValues,hhData,ft_hog,h,N,u,v);

		for(j=0;j<N;j++)
			hog_data_values[(j*n_fts) +i] = h[j];
	}

	free(h);
	free(ft_hog);

	return;
}


void getRectangleSingleHoGFeature(double * rect, mxArray * hhData,double * ft_hog, double * h, int N, int u , int v)
{
// hh : histogram integrale
// ft_HoG
// ws : window size
	double * hh;
	int i,wnd_size,pas,x,y,dx,x1,x2,x3,x4,type,mx;
	double sc=ft_hog[2];
	double cum_sum = 0;

    mx = u * v - 1;
	wnd_size = 32;
	pas = rect[2] / wnd_size;
	type = ft_hog[1];
// Feature 
	x=rect[1]-1+(ft_hog[3]-1)*pas;
	y=rect[0]-1+(ft_hog[4]-1)*pas;
	dx = sc * pas;

	hh = mxGetPr(hhData);
	
	switch(type)
	{
		case 1:
			x1 = y + x*u;
			x2 = y + (x+dx)*u;
			x3 = (y+dx) + (x+dx)*u;
			x4 = (y+dx) + x*u;

			break;
		case 2:
			x1 = y + x*u;
			x2 = y + (x+2*dx)*u;
			x3 = (y+dx) + (x+2*dx)*u;
			x4 = (y+dx) + x*u;

			break;
		case 3:
			x1 = y + x*u;
			x2 = y + (x+dx)*u;
			x3 = (y+2*dx) + (x+dx)*u;
			x4 = (y+2*dx) + x*u;

			break;
	}

	if ( x1 > mx )
	{
		mexPrintf("valor x1: %i y max: %i\n",x1,mx);
		x1 = mx;
	}
	if ( x2 > mx )
	{
		mexPrintf("valor x1: %i y max: %i\n",x2,mx);
		x2 = mx;
	}
	if ( x3 > mx )
	{
		mexPrintf("valor x1: %i y max: %i\n",x3,mx);
		x3 = mx;
	}
	if ( x4 > mx )
	{
		mexPrintf("valor x1: %i y max: %i\n",x4,mx);
		x4 = mx;
	}

	for(i=0;i<N;i++) 
		h[i] = fabs((hh[u*v*i + x1] + hh[u*v*i + x3]) - 
			(hh[u*v*i + x2] + hh[u*v*i + x4]));

	for(i=0;i<N;i++)
	{
		if(h[i] < 1)
			h[i] = 0;
		cum_sum += h[i];
	}

	if(cum_sum)
	{
		for(i=0;i<N;i++)
			h[i] = h[i] / cum_sum;
	}
	return;
}
