function script_get_Haar_index
% Calculate the linear Haar index to (x,y) format:
% For any Haar feature -> their position (x,y) in the original image.
% f1:   -1 | +1
%
% f2:   -1
%       --
%       +1
%
% f3: +1 | -2 | +1
%
% f4:   +1
%       --
%       -2
%       --
%       +1
%
% The output file has 5 columns and each of them means:
% 1-	feature index number.
% 2-	the type of the Haar feature (f1 and f2).
% 3-	The scale of the feature.
% 4-	The x position (assuming the patch has size 32x32 pixels).
% 5-	The y position.

nro_feat = 2;
% Our original size is the 32x32 window
u=32;v=32;
index_Haar = [];
acc_index = 1;
for sc=[4 6 8]
    q_w = v - 2*sc;
    q_h = u - sc;

    % Feature 1
    for i=1:q_w, % colonnes
        for j=1:q_h, % lines
            index_Haar = [index_Haar ; acc_index 1 sc i j];  
            acc_index = acc_index+1;
        end
    end
    q_w = v - sc;
    q_h = u - 2*sc;
    % Feature 2
    for i=1:q_w,
        for j=1:q_h,
            index_Haar = [index_Haar ; acc_index 2 sc i j];  
            acc_index = acc_index+1;
        end
    end
    % Feature 3
    q_w = v - 3*sc;
    q_h = u - sc;
    for i=1:q_w,
        for j=1:q_h,
            index_Haar = [index_Haar ; acc_index 3 sc i j];  
            acc_index = acc_index+1;
        end
    end
    % Feature 4
    q_w = v - sc;
    q_h = u - 3*sc;
    for i=1:q_w,
        for j=1:q_h,
            index_Haar = [index_Haar ; acc_index 4 sc i j];  
            acc_index = acc_index+1;
        end
    end

end
save('index_Haar','index_Haar');
save('..\TrainBoost\index_Haar','index_Haar');
save('..\Detection\index_Haar','index_Haar');
