%%%%%%%%%%%%%%%% v 3.0 - Real Adaboost %%%%%%%%%%%%%%%%%%%%%%
function trainCascadeDetector

global HAAR_TYPE HOG_TYPE NROBINS WIND_SIZE NROMAXNEG;

HAAR_TYPE   = 0;
HOG_TYPE    = 1;
NROBINS     = 128;
PN_FACTOR_MUL_THRESHOLD	=	0.998;
PN_MAX_IT_THRESHOLD		=	1500;
WIND_SIZE   = 32;
NROMAXNEG   = 1000;
SAVE_NEGATIVES = 0;
addpath('..\Functions');
addpath('..\Functions\c');
disp('lecture des images non-vehicule');
neg_base_path = '..\Images\Negatives';
% Initial values
rand('seed',1);
% Variables definitions
f_mx=0.4; % maximum acceptable false positive rate
d_mn=0.95; % minimum acceptable detection rate per layer 
F(1)=1.0;D(1)=1.0;
Ftarget=0.000001;
C = [];
N = 4;
MAX_STAGES=20;
% Load positive features
load vehicule_positive_32;
% split into training and validation sets
[haar_nrofts,NROPOS]=size(P_haar);
[hog_nrofts,pipo]=size(P_hog);
NROTRAIN  = round(NROPOS * 0.7);
NVAL    = NROPOS - NROTRAIN;
% get random index
x = randperm(NROPOS);

% Haar
PHaar_v = P_haar(:,x(1:NVAL));
PHaar_t = P_haar(:,x(NVAL+1:end));

% HoG
PHoG_v = zeros(hog_nrofts,NVAL*N);
PHoG_t = zeros(hog_nrofts,NROTRAIN*N);
for i=1:NVAL
    idx = 1 + N * (x(i) - 1);
    id = 1 + N * (i - 1);
    PHoG_v(:,id:id+N-1) = P_hog(:,idx:idx+N-1);
end
for i=1:NROTRAIN
    idx = 1 + N * (x(NVAL+i) - 1);
    id = 1 + N * (i - 1);
    PHoG_t(:,id:id+N-1) = P_hog(:,idx:idx+N-1);
end

clear P_haar P_hog;

% Compute HoG model:
HoG_model = computeHoGModel(PHoG_t,N);
% Get HoG Battacharyya distances
PHoG_v = getBaseHistogramDistances(PHoG_v,HoG_model,N);
PHoG_t = getBaseHistogramDistances(PHoG_t,HoG_model,N);

i=1;

version = '0';
%version = input('Input version: ','s');
file_str = ['train_' version '.dat'];
it = 1;
while F(i) > Ftarget
    i    
    i=i+1;
    n=0;
    F(i)=F(i-1);
    D(i)=D(i-1);
    pause(1);
    hj=[];
    pd = 0; % poids

    rand('seed',i);
    
    file_neg = ['neg_data_' num2str(it) '.mat'];
    fid = fopen(file_neg,'r');
    if fid < 0
        [NHaar,NHoG] = getNegativeSamples(neg_base_path,C,N);
        if size(NHaar,2) < NROTRAIN
            disp('Stop learning for insuficient negatives');
            break;
        end
        if SAVE_NEGATIVES
            save(file_neg,'NHaar','NHoG');
        end
    else
        load(file_neg);
        fclose(fid);
    end

    
    NHoG = getBaseHistogramDistances(NHoG,HoG_model,N);

    while F(i) > (f_mx * F(i-1)) || D(i) < (d_mn * D(i-1))
        n=n+1;
        [hj,pd] = realadaboost_fm(NHaar,NHoG,PHaar_t,PHoG_t,hj,n,pd,HoG_model,N);
        % change the threshold to have a rate of d_mn positifs 
        low = computeLowThresholdValue(hj);
        high = computeHighThresholdValue(hj);
        dx = high - low;
        j = 0;
        TH = PN_FACTOR_MUL_THRESHOLD^j;
        while (computeDetectionRatio(hj,PHaar_v,PHoG_v,TH) < d_mn) && j < PN_MAX_IT_THRESHOLD
            j = j + 1;
            TH = PN_FACTOR_MUL_THRESHOLD^j * dx + low;
        end
        d = computeDetectionRatio(hj,PHaar_v,PHoG_v,TH);
        f = computeDetectionRatio(hj,NHaar,NHoG,TH);
        disp(['d: ' num2str(d*100) ' % , f: ' num2str(f*100) ' , for ' num2str(j) ' iterations']);
        D(i)=D(i-1)*d;
        F(i)=F(i-1)*f;
        disp(['Evaluation -> D: ' num2str(D(i)) ' % , F: ' num2str(F(i)) ]);
        
        if isOutLimitFeaturesMaxStage(i-1,n)
            break;
        end
    end
    C{it}.hj = hj;  
    C{it}.threshold = TH;
    
    save('CascadeDetector','C');
    % get the false positive from the negative base
    if it==MAX_STAGES
        break;
    end
    clear NHaar NHoG;
    it = it + 1;
end
save('..\Detection\CascadeDetector','C');


function val = computeLowThresholdValue(hj)
[nn,pp]=size(hj);
val = 0;
for i=1:pp
    val = val + min(hj{i}.fm(:));
end
function val = computeHighThresholdValue(hj)
[nn,pp]=size(hj);
val = 0;
for i=1:pp
    val = val + max(hj{i}.fm(:));
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% function d=computeDetectionRatio(hj,Haar_v,HoG_v,TH)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
function d = computeDetectionRatio(hj,FHaar,FHoG,TH)

global HAAR_TYPE HOG_TYPE NROBINS;

[pipo,nsamples] = size(FHaar);
[pipo,nfeatures] = size(hj);

acc_sum = zeros(1,nsamples);

for i=1:nfeatures
    switch hj{i}.type
        case HAAR_TYPE
            fts = FHaar(hj{i}.feature,:);
            idx = floor((fts / hj{i}.maxval) * NROBINS) + 1;
            % verify values
            x = find(idx > NROBINS);
            if ~isempty(x), idx(x)=NROBINS;end
            acc_sum = acc_sum + hj{i}.fm(idx);
        case HOG_TYPE
            fts = FHoG(hj{i}.feature,:);
            idx = floor(fts * NROBINS) + 1;
            % verify values
            x = find(idx > NROBINS);
            if ~isempty(x), idx(x)=NROBINS;end         
            acc_sum = acc_sum + hj{i}.fm(idx);
    end
end
x = find(acc_sum >= TH);
d = length(x) / nsamples;
            
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   FUNCTION GET NEGATIVE IMAGES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [NHaar,NHoG] = getNegativeSamples(base_path,C,N)
% We will go to get 1500 false positives from the BA
% We have 10 folders with images
global NROMAXNEG;

data = [];
load index_Haar;
load index_HoG;
NHaar = [];
NHoG = [];

stage_path = [base_path '\'];
% lecture des fichiers d'image
fichiers = [dir([stage_path '\*.jpg']);dir([stage_path '\*.bmp']);dir([stage_path '\*.ppm'])...
        ;dir([stage_path '\*.png']);dir([stage_path '\*.tif']);dir([stage_path '\*.jpeg'])];
[nro_files,pipo]=size(fichiers);
% for 600 fixed false positives, we have nro_wind for each image in the
% folder:
nro_wind = round(NROMAXNEG / nro_files);
acc_winds = 0;
% the negatives files will be extracted in random way
x = randperm(nro_files);

for i=1:nro_files,
    nro_scales = 7;
    img = imread([stage_path  '\' fichiers(x(i)).name]);
    [u,v,d]=size(img);
    if d > 1 
        img=rgb2gray(img);
    end
    img = double(img);
    pause(0.00001);
    disp(['Negative file: ' fichiers(x(i)).name]);     
    % look for the image size and the number of scales
    while min([u v])/(32*nro_scales) <= 1
        nro_scales = nro_scales - 1;
        if nro_scales == 0
            nro_scales=1;
            break;
        end
    end
    [haar,hog,wind] = getFalseAlarms(img,nro_scales,nro_wind,C,index_Haar,index_HoG,N);   
    
    acc_winds = acc_winds + wind;
    disp(['Negatives samples founded : ' num2str(wind) ' (Tot : ' num2str(acc_winds) ')']);
    NHaar = [NHaar haar];
    NHoG = [NHoG hog];
    
    if acc_winds >= NROMAXNEG
        break;
    end
    
    nro_wind = max([3 ceil((NROMAXNEG-acc_winds) / (nro_files-i+1))]);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function  [data,wind] = getFalseAlarms(img,nro_scales,nro_wind,C,TH,index_Haar)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [haar,hog,wind] = getFalseAlarms(img,nro_scales,nro_wind,C,index_Haar,index_HoG,N)

[u,v,d]=size(img);

NROHAARFTS  = size(index_Haar,1);
NROHOGFTS   = size(index_HoG,1);

ii = imageIntegrale(img);
[m,o]=getgradsobel(img,N);
% clean bords
m(1,:)=0;m(end,:)=0;
m(:,1)=0;m(:,end)=0;
% calculate the histogram integral
hh = getHistogramIntegral(m,o,N);

% redimensionner la matrice
iii = [zeros(1,v+1); zeros(u,1) ii];
for i=1:N
    hhh(:,:,i) = [zeros(1,v+1); zeros(u,1) hh(:,:,i)];
end
% for standard deviation calcul
img2 = img.^2;
ii2 = imageIntegrale(img2);
iii2 = [zeros(1,v+1); zeros(u,1) ii2];

wind=0;
tot_expected = nro_scales*nro_wind;
haar = zeros(NROHAARFTS,tot_expected);
hog = zeros(NROHOGFTS,N*tot_expected);

for k=1:nro_scales
    pause(0.002);
    i=1;n=0;
    h = randperm(u-k*32);
    c = randperm(v-k*32);
    mx_tst = min([size(h,2) size(c,2)]);
    while i < mx_tst && n < nro_wind
    	y = h(i);x = c(i);dxy = 32*k;
        rect = [y x dxy];
        
        % compute mean of the rectangle pixels gray values
        s = getRectangleStandardDeviation(rect,iii,iii2);
        if validateSubImage(iii,hhh,rect,s,C,index_Haar,index_HoG)
            wind = wind + 1;
            haar(:,wind) = getHaarData_c(rect,iii,index_Haar,s);
            idx = 1 + N * (wind - 1);
            hog(:,idx:idx+N-1) = getHoGData_c(rect,hhh,N,index_HoG);
            n = n + 1;
        end
        i=i+1;
    end
end
if tot_expected > wind
    haar(:,wind+1:end) = [];
    idx = 1 + N * wind;
    hog(:,idx:end) = [];
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function rsp = validateSubImage(ii,hh,rect,s,C,index_Haar,index_HoG)

rsp = 1;

[pipo,nstg]=size(C);
for s=1:nstg,
    hj = C{s}.hj;
    acc_sum = detectionOnRectangle(rect,ii,hh,s,hj,index_Haar,index_HoG);
    if acc_sum < C{s}.threshold
        rsp = 0;
        break;
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function rsp = isOutLimitFeaturesMaxStage(stage,nfeatures)
rsp = 0;
% Exponential law
a = 3.026;
b = 0.2096;
val = floor(a * exp(b * stage));
if val > 200
    val = 200;
end
if nfeatures >= val
    rsp = 1;
end

